chai = require('chai')
expect = chai.expect;

predicateFrom = require('./index.js')


describe('predicateFrom', function () {
  let isEven = function (x) { return x % 2 === 0 }
  it("should leave a function untouched", () => {
    expect(predicateFrom(isEven)).to.be.a.function;
    expect(predicateFrom(isEven)).to.equal(isEven);
    expect(predicateFrom(isEven)(4)).to.be.true;
    expect(predicateFrom(isEven)(3)).to.be.false;
  });
  it("should convert a string to a predicate", () => {
    expect(predicateFrom('a')).to.be.a.function;
    expect(predicateFrom('a')("a")).to.be.true;
    expect(predicateFrom('a')("aa")).to.be.false;
  });
  it("should convert a number to a predicate", () => {
    expect(predicateFrom(1)).to.be.a.function;
    expect(predicateFrom(1)(1)).to.be.true;
    expect(predicateFrom(1)(2)).to.be.false;
  });
  it("should convert a boolean to a predicate", () => {
    expect(predicateFrom(true)).to.be.a.function;
    expect(predicateFrom(true)(true)).to.be.true;
    expect(predicateFrom(true)(false)).to.be.false;
    expect(predicateFrom(false)(false)).to.be.true;
    expect(predicateFrom(false)(true)).to.be.false;
  });
});
describe("predicates from arrays", () => {
  it("should convert an array to a or", () => {
    expect(predicateFrom(['a','c'])('a')).to.be.true
    expect(predicateFrom(['a','c'])('b')).to.be.false
  });
  it("should convert special arrays", () => {
    // empty arrays
    expect(predicateFrom([])('a')).to.be.true
    // single item arrays
    expect(predicateFrom(['a'])('a')).to.be.true
    expect(predicateFrom(['a'])('b')).to.be.false
  });
  it("should support array of arrays", () => {
    let pf = predicateFrom([ ['a','b'],['c','d']]);
    expect(pf('a')).to.be.true;
    expect(pf('e')).to.be.false;
  });
  it("should support array of functions", () => {
    let pf = predicateFrom([ x => x.endsWith('.js'),x => x.endsWith('.jsx')]);
    expect(pf('index.js')).to.be.true;
    expect(pf('index.jsx')).to.be.true;
    expect(pf('index.js.bak')).to.be.false;
  });
});
describe("predicate from objects", () => {
  let teams = [{ name:"PSG", league:1},{ name:"OM", league:1},{ name:"TFC", league:2}]
  let cities = [{ city:"Paris", team: { name:"PSG", league:1} } ]
  describe("with native properties", () => {
    expect(predicateFrom({ a: 1 })).to.be.a.function;
    expect(predicateFrom({ a: 1 })({ a: 1 })).to.be.true;
    expect(predicateFrom({ a: 1 })({ a: 1, b: 2 })).to.be.true;
    expect(predicateFrom({ a: 1 })({ a: 2 })).to.eq(false);
  });
});